@@ -1,10 +1,11 @@
 ###
-### $Release: 0.9.0 $
-### copyright(c) 2007-2010 kuwata-lab.com all rights reserved.
+### $Release: 1.0.0 $
+### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
 ###
 
-from oktest import ok, not_ok, run
-import sys, os, re, time, marshal
+from oktest import ok, not_ok, run, spec
+from oktest.helper import dummy_file
+import sys, os, re, time, marshal, shutil
 from glob import glob
 try:    import cPickle as pickle
 except: import pickle
@@ -14,6 +15,8 @@ import tenjin
 #from tenjin.helpers import escape, to_str
 from tenjin.helpers import *
 
+lvars = "_extend=_buf.extend;_to_str=to_str;_escape=escape; "
+
 
 def _convert_data(data, lang='python'):
     if isinstance(data, dict):
@@ -40,6 +43,51 @@ def _remove_files(basenames=[]):
             os.unlink(filename)
 
 
+def _with_dummy_files(func):
+    """decorator for test functions"""
+    def deco(self):
+        isdir = os.path.isdir
+        mkdir = os.mkdir
+        try:
+            if not isdir('_views'): mkdir('_views')
+            if not isdir('_views/blog'): mkdir('_views/blog')
+            pairs = [ ('_views/blog/index.pyhtml', 'xxx'),
+                      ('_views/index.pyhtml', '<<#{{_DUMMY_VALUE}}>>'),
+                      ('_views/layout.pyhtml', '<div>#{_content}</div>'), ]
+            for fname, cont in pairs:
+                f = open(fname, 'w')
+                f.write(cont)
+                f.close()
+            func(self)
+        finally:
+            shutil.rmtree('_views')
+    for x in ('func_name', '__name__', '__doc__'):
+        if hasattr(func, x):
+            setattr(deco, x, getattr(func, x))
+    return deco
+
+
+class DebugLogger(object):
+
+    def __init__(self):
+        self.messages = []
+
+    def __str__(self):
+        return "\n".join(self.messages)
+
+    def f(name):
+        def func(self, format, *args):
+            msg = args and (format % args) or format
+            self.messages.append("[%s] %s" % (name.upper(), msg))
+        func.func_name = func.__name__ = name
+        return func
+
+    fatal, error, warn, info, debug, trace = \
+        f('fatal'), f('error'), f('warn'), f('info'), f('debug'), f('trace')
+
+    del f
+
+
 class EngineTest(object):
 
     #code = TestCaseHelper.generate_testcode(__file__)
@@ -190,6 +238,48 @@ class EngineTest(object):
             _remove_files([layout_filename, content_filename])
 
 
+    def test_capture_as(self):
+        filename  = '.test.capture_as.pyhtml'
+        content1  = ("<?py from __future__ import with_statement ?>\n"
+                     "<div>\n"
+                     "<?py with capture_as('sidemenu'): ?>\n"
+                     "  <ul>\n"
+                     "  <?py for item in items: ?>\n"
+                     "    <li>${item}</li>\n"
+                     "  <?py #endfor ?>\n"
+                     "  </ul>\n"
+                     "<?py #endwith ?>\n"
+                     "</div>\n")
+        content2  = ("<div>\n"
+                     "<?py for _ in capture_as('sidemenu'): ?>\n"
+                     "  <ul>\n"
+                     "  <?py for item in items: ?>\n"
+                     "    <li>${item}</li>\n"
+                     "  <?py #endfor ?>\n"
+                     "  </ul>\n"
+                     "<?py #endfor ?>\n"
+                     "</div>\n")
+        expected1 = ("<div>\n"
+                     "</div>\n")
+        expected2 = ("  <ul>\n"
+                     "    <li>A</li>\n"
+                     "    <li>B</li>\n"
+                     "    <li>C</li>\n"
+                     "  </ul>\n")
+        contents = (content1, content2)
+        if sys.version < '2.5':
+            contents = (content2, )
+        for content in contents:
+            d = dummy_file(filename, content).__enter__()
+            try:
+                context = {'items': ['A', 'B', 'C']}
+                engine = tenjin.Engine()
+                output = engine.render(filename, context)
+                ok (output) == expected1
+                ok (context['sidemenu']) == expected2
+            finally:
+                d.__exit__(*sys.exc_info())
+
     def test_captured_as(self):
         hash = EngineTest.testdata['test_captured_as']
         files = ( ('content.pyhtml',      hash['content']),
@@ -388,10 +478,11 @@ class EngineTest(object):
             engine = tenjin.Engine(cache=storage)
             engine.render(template_name)
             fullpath = os.path.abspath(template_name)
-            ok (storage.items).contains(fullpath)
+            cachepath = engine.cachename(fullpath)
+            ok (storage.items).contains(cachepath)
             ok (template_name + '.cache').exists()
-            storage.unset(fullpath)
-            not_ok (storage.items).contains(fullpath)
+            storage.unset(cachepath)
+            not_ok (storage.items).contains(cachepath)
             not_ok (template_name + '.cache').exists()
             def f():
                 storage.unset(fullpath)
@@ -661,5 +752,212 @@ class EngineTest(object):
             not_ok ('foo.pyhtml').exists()
 
 
+    #def test_prefer_fullpath(self):
+    #    fname = 'test_prefer_fullpath.pyhtml'
+    #    input = ( "<?py for x in items: ?>\n"
+    #              "  <p>${x.length}</p>\n"
+    #              "<?py #end ?>\n" )
+    #    context = {'items': ['A']}
+    #    engine = None
+    #    #
+    #    def _test(fname, input, expected):
+    #        write_file(fname, input)
+    #        engine = tenjin.Engine()
+    #        engine.cache.clear()
+    #        t = engine.get_template(fname)
+    #        ok (t.filename) == expected
+    #        ex = error_file = None
+    #        try:
+    #            engine.render(fname, context)
+    #        except:
+    #            ex = sys.exc_info()[1]
+    #            error_file = _filename_on_where_error_raised()
+    #        ok (ex) != None
+    #        ok (error_file) == expected
+    #        # read from cache
+    #        engine.cache.clear()
+    #        engine = tenjin.Engine()
+    #        t = engine.get_template(fname)
+    #        ok (t.filename) == expected
+    #    def _filename_on_where_error_raised():
+    #        import traceback
+    #        tb = sys.exc_info()[2]
+    #        arr = traceback.format_tb(tb)
+    #        msg = arr[-1]
+    #        m = re.search(r'File "(.*?)"', msg)
+    #        return m.group(1)
+    #    #
+    #    if "Engine.prefer_fullpath is False then set relative epath as template filename":
+    #        assert tenjin.Engine.prefer_fullpath == False
+    #        try:
+    #            _test(fname, input, expected=fname)
+    #        finally:
+    #            _remove_files([fname, fname + '.cache'])
+    #    if "Engine.prefer_fullpath is True then set fullpath as template filename":
+    #        tenjin.Engine.prefer_fullpath = True
+    #        try:
+    #            _test(fname, input, expected=os.path.join(os.getcwd(), fname))
+    #        finally:
+    #            tenjin.Engine.prefer_fullpath = False
+    #            _remove_files([fname, fname + '.cache'])
+
+
+    ##################
+
+    def test_cachename(self):
+        engine = tenjin.Engine()
+        if spec("return cache file path"):
+            ok (engine.cachename('foo.pyhtml')) == 'foo.pyhtml.cache'
+
+    def test_to_filename(self):
+        engine = tenjin.Engine(prefix='user_', postfix='.pyhtml')
+        if spec("if template_name starts with ':', add prefix and postfix to it."):
+            ok (engine.to_filename(':list')) == 'user_list.pyhtml'
+        if spec("if template_name doesn't start with ':', just return it."):
+            ok (engine.to_filename('list')) == 'list'
+
+    @_with_dummy_files
+    def test__create_template(self):
+        e1 = tenjin.Engine(path=['_views/blog', '_views'])
+        t = None
+        if spec("if input is not specified then just create empty template object."):
+            t = e1._create_template(None)
+            ok (t).is_a(tenjin.Template)
+            ok (t.filename) == None
+            ok (t.script) == None
+        if spec("if input is specified then create template object and return it."):
+            t = e1._create_template('<p>#{_content}</p>', '_views/layout.pyhtml')
+            ok (t).is_a(tenjin.Template)
+            ok (t.filename) == "_views/layout.pyhtml"
+            ok (t.script) == lvars + "_extend(('''<p>''', _to_str(_content), '''</p>''', ));"
+
+    @_with_dummy_files
+    def test__preprocess(self):
+        e1 = tenjin.Engine()
+        if spec("preprocess template and return result"):
+            fpath = '_views/index.pyhtml'
+            input, mtime = e1.loader.load(fpath)
+            ret = e1._preprocess(input, fpath, {}, globals())
+            ok (ret) == "<<SOS>>"
+
+    @_with_dummy_files
+    def test__get_template_from_cache(self):
+        e1 = tenjin.Engine(path=['_views/blog', '_views'], postfix='.pyhtml')
+        fpath = '_views/blog/index.pyhtml'
+        cpath = fpath + '.cache'
+        if spec("if template not found in cache, return None"):
+            ret = e1._get_template_from_cache(cpath, fpath)
+            ok (ret) == None
+        t = tenjin.Template(fpath)
+        e1.cache.set(fpath + '.cache', t)
+        if spec("if checked within a sec, skip timestamp check."):
+            #try:
+            #    t.timestamp = time.time() - 0.2
+            #    #import pdb; pdb.set_trace()
+            #    tenjin.logger = DebugLogger()
+            #    ret = e1._get_template_from_cache(cpath, fpath)
+            #    msg = tenjin.logger.messages[0]
+            #    ok (msg.startswith("[TRACE] [tenjin.Engine] timestamp check skipped (")) == True
+            #finally:
+            #    tenjin.logger = None
+            pass
+        if spec("if timestamp of template objectis same as file, return it."):
+            t._last_checked_at = None
+            t.timestamp = os.path.getmtime(fpath)
+            ok (e1._get_template_from_cache(cpath, fpath)).is_(t)
+            ok (t._last_checked_at).in_delta(time.time(), 0.001)
+        if spec("if timestamp of template object is different from file, clear it"):
+            t.timestamp = t.timestamp + 1
+            t._last_checked_at = None
+            try:
+                #import pdb; pdb.set_trace()
+                tenjin.logger = DebugLogger()
+                ret = e1._get_template_from_cache(cpath, fpath)
+                msg = tenjin.logger.messages[0]
+                ok (msg) == "[INFO] [tenjin.Engine] cache expired (filepath='_views/blog/index.pyhtml')"
+            finally:
+                tenjin.logger = None
+
+    @_with_dummy_files
+    def test_get_template(self):
+        e1 = tenjin.Engine(path=['_views/blog', '_views'], postfix='.pyhtml')
+        filepath  = '_views/blog/index.pyhtml'
+        fullpath  = os.getcwd() + filepath
+        cachepath = fullpath + '.cache'
+        assert not os.path.exists(cachepath)
+        t = None
+        if spec("return template object.") and \
+           spec("accept template_name such as ':index'."):
+            t = e1.get_template(':index')
+            ok (t).is_a(tenjin.Template)
+            ok (t.filename) == filepath
+        if spec("if template object is added by add_template(), return it."):
+            tmp = tenjin.Template('foo.pyhtml', input="<<dummy>>")
+            e1.add_template(tmp)
+            ok (e1.get_template('foo.pyhtml')).is_(tmp)
+        if spec("get filepath and fullpath of template"):
+            e1._filepaths['index.pyhtml'] == (filepath, fullpath)
+        if spec("if template file is not found then raise TemplateNotFoundError"):
+            def f(): e1.get_template('index')
+            ok (f).raises(tenjin.TemplateNotFoundError, "index: filename not found (path=['_views/blog', '_views']).")
+        if spec("use full path as base of cache file path") and \
+           spec("get template object from cache"):
+            ok (list(e1.cache.items.keys())) == ["%s/_views/blog/index.pyhtml.cache" % os.getcwd()]
+        if spec("change template filename according to prefer_fullpath"):
+            pass
+        if spec("use full path as base of cache file path"):
+            pass
+        if spec("get template object from cache"):
+            pass
+        if spec("if template object is not found in cache or is expired..."):
+            e1.cache.clear()
+            ok (len(e1.cache.items)) == 0
+            tname = ':layout'
+            fpath = '_views/layout.pyhtml'
+            cpath = os.path.join(os.getcwd(), '_views/layout.pyhtml.cache')
+            not_ok (cpath).exists()
+            if spec("create template object."):
+                t = e1.get_template(tname)
+                ok (t).is_a(tenjin.Template)
+            if spec("set timestamp and filename of template object."):
+                ok (t.timestamp) == os.path.getmtime(filepath)
+                ok (t.filename) == fpath
+                ok (t._last_checked_at).in_delta(time.time(), 0.005)
+            if spec("save template object into cache."):
+                ok (cpath).exists()
+                ok (len(e1.cache.items)) == 1
+                ok (e1.cache.items).contains(cpath)
+
+    def test_include(self):
+        if spec("get local and global vars of caller."):
+            pass
+        if spec("get _context from caller's local vars."):
+            pass
+        if spec("if kwargs specified then add them into context."):
+            pass
+        if spec("get template object with context data and global vars."):
+            pass
+        if spec("if append_to_buf is true then add output to _buf."):
+            pass
+        if spec("if append_to_buf is false then don't add output to _buf."):
+            pass
+        if spec("render template and return output."):
+            pass
+        if spec("kwargs are removed from context data."):
+            pass
+
+    def test_hook_context(self):
+        e = tenjin.Engine()
+        ctx = {}
+        e.hook_context(ctx)
+        if spec("add engine itself into context data."):
+            ok (ctx.get('_engine')).is_(e)
+        if spec("add include() method into context data."):
+            ok (ctx.get('include')) == (e.include)
+
+
+_DUMMY_VALUE = 'SOS'
+
+
 if __name__ == '__main__':
     run(EngineTest)
