@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ###
-### $Release: 0.9.0 $
-### copyright(c) 2007-2010 kuwata-lab.com all rights reserved.
+### $Release: 1.0.0 $
+### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
 ###
 
 from oktest import ok, not_ok, run
@@ -22,7 +22,6 @@ __name__ = 'dummy'
 exec(tenjin._read_binary_file(filename).decode('utf-8'))
 __name__ = _name_orig
 
-
 def to_list(value):
     if isinstance(value, list):
         return value
@@ -59,28 +58,28 @@ INPUT3 = r"""<?py
 </ul>
 """
 
-SOURCE = r"""_buf = []; _buf.extend(('''<ul>\n''', ));
+SOURCE = r"""_buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; _extend(('''<ul>\n''', ));
 for item in ['<a&b>', '["c",'+"'d']"]:
-    _buf.extend(('''  <li>''', to_str(item), '''
-      ''', escape(to_str(item)), '''</li>\n''', ));
+    _extend(('''  <li>''', _to_str(item), '''
+      ''', _escape(_to_str(item)), '''</li>\n''', ));
 #end
-_buf.extend(('''</ul>\n''', ));
+_extend(('''</ul>\n''', ));
 print(''.join(_buf))
 """
-SOURCE2 = """_buf = []; _buf.extend(('''<ul>\\r\\n''', ));\n\
+SOURCE2 = """_buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; _extend(('''<ul>\\r\\n''', ));\n\
 for item in ['<a&b>', '["c",'+"'d']"]:\n\
-    _buf.extend(('''  <li>''', to_str(item), '''\r\n\
-      ''', escape(to_str(item)), '''</li>\\r\\n''', ));\n\
+    _extend(('''  <li>''', _to_str(item), '''\r\n\
+      ''', _escape(_to_str(item)), '''</li>\\r\\n''', ));\n\
 #end\n\
-_buf.extend(('''</ul>\\r\\n''', ));\n\
+_extend(('''</ul>\\r\\n''', ));\n\
 print(''.join(_buf))
 """
-SOURCE_N = r"""    1:  _buf = []; _buf.extend(('''<ul>\n''', ));
+SOURCE_N = r"""    1:  _buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; _extend(('''<ul>\n''', ));
     2:  for item in ['<a&b>', '["c",'+"'d']"]:
-    3:      _buf.extend(('''  <li>''', to_str(item), '''
-    4:        ''', escape(to_str(item)), '''</li>\n''', ));
+    3:      _extend(('''  <li>''', _to_str(item), '''
+    4:        ''', _escape(_to_str(item)), '''</li>\n''', ));
     5:  #end
-    6:  _buf.extend(('''</ul>\n''', ));
+    6:  _extend(('''</ul>\n''', ));
     7:  print(''.join(_buf))
 """
 
@@ -88,14 +87,14 @@ EXECUTED = r"""<ul>
   <li><a&b>
       &lt;a&amp;b&gt;</li>
   <li>["c",'d']
-      [&quot;c&quot;,'d']</li>
+      [&quot;c&quot;,&#39;d&#39;]</li>
 </ul>
 """
 EXECUTED2 = """<ul>\r\n\
   <li><a&b>\r\n\
       &lt;a&amp;b&gt;</li>\r\n\
   <li>["c",'d']\r\n\
-      [&quot;c&quot;,'d']</li>\r\n\
+      [&quot;c&quot;,&#39;d&#39;]</li>\r\n\
 </ul>\r\n\
 """
 EXECUTED3 = r"""<h1>tenjin example</h1>
@@ -185,8 +184,8 @@ class MainTest(object):
                     ok (str(ex)) == errormsg
             else:
                 output = app.execute()
-                #print "*** expected=%s" % repr(expected)
-                #print "*** output=%s" % repr(output)
+                #print "*** expected=%s" % expected
+                #print "*** output=%s" % output
                 if python2:
                     if encoding and isinstance(output, unicode):
                         output = output.encode(encoding)
@@ -253,7 +252,7 @@ class MainTest(object):
         n2 = len("</ul>\n")
         self.input    = INPUT[n1:-n2]
         buf = SOURCE.splitlines(True)[1:-2]
-        buf.insert(0, "_buf = []\n")
+        buf.insert(0, "_buf = []\n_extend=_buf.extend;_to_str=to_str;_escape=escape; \n")
         buf.append("print(''.join(_buf))\n")
         self.expected = ''.join(buf)
         self._test()
@@ -296,12 +295,12 @@ class MainTest(object):
         self.expected = ''
         script = (
             "title = _context.get('title'); items = _context.get('items'); \n"
-            "_buf.extend(('''<h1>''', escape(to_str(title)), '''</h1>\n"
+            "_extend=_buf.extend;_to_str=to_str;_escape=escape; _extend(('''<h1>''', _escape(_to_str(title)), '''</h1>\n"
             "<ul>\\n''', ));\n"
             "for item in items:\n"
-            "    _buf.extend(('''  <li>''', escape(to_str(item)), '''</li>\\n''', ));\n"
+            "    _extend(('''  <li>''', _escape(_to_str(item)), '''</li>\\n''', ));\n"
             "#endfor\n"
-            "_buf.extend(('''</ul>\\n''', ));\n"
+            "_extend(('''</ul>\\n''', ));\n"
             )
         self.filename = 'test_cache1.pyhtml'
         cachename = self.filename + '.cache'
@@ -328,22 +327,22 @@ class MainTest(object):
         '        <th>#</th><th>item</th>\n'
         '      </tr>\n'
         '    </thead>\n'
-        '    </tbody>\n'
         '    <?py i = 0 ?>\n'
         '    <?py for item in list: ?>\n'
         '\t<?py i += 1 ?>\n'
+        '    <tbody>\n'
         '      <tr bgcolor="#{i % 2 and "#FFCCCC" or "#CCCCFF"}">\n'
         '\t<td>${i}</td>\n'
         '        <td>${item}</td>\n'
         '      </tr>\n'
-        '    <?py #end ?>\n'
         '    </tbody>\n'
+        '    <?py #end ?>\n'
         '  </table>\n'
         '<?py #end ?>'
         '</div>\n'
         )
     expected_for_retrieve = '\n'.join((
-        '_buf = []; ',
+        '_buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; ',
         'if list:',
         '',
         '',
@@ -351,44 +350,47 @@ class MainTest(object):
         '',
         '',
         '',
-        '',
         '    i = 0',
         '    for item in list:',
-        '\ti += 1',
-        '                   to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
-        '\t    escape(to_str(i)); ',
-        '            escape(to_str(item)); ',
+        '        i += 1',
+        '',
+        '        _to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
+        '        _escape(_to_str(i)); ',
+        '        _escape(_to_str(item)); ',
+        '',
         '',
         '    #end',
         '',
-        '',
         '#end',
+        '',
         'print(\'\'.join(_buf))',
         ''))
 
-    def test_retrieve(self):  # -S, -a retrieve
+    def test_retrieve1(self):  # -S, -a retrieve
         self.input    = self.input_for_retrieve
         self.expected = self.expected_for_retrieve
         self.options = '-S'
         self._test()
         self.options = '-a retrieve'
-        self._test()
+        #self._test()
 
     def test_retrieve2(self):  # -SU, -SNU
         expected = '\n'.join((
-            '_buf = []; ',
+            '_buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; ',
             'if list:',
             '',
             '    i = 0',
             '    for item in list:',
-            '\ti += 1',
-            '                   to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
-            '\t    escape(to_str(i)); ',
-            '            escape(to_str(item)); ',
+            '        i += 1',
+            '',
+            '        _to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
+            '        _escape(_to_str(i)); ',
+            '        _escape(_to_str(item)); ',
             '',
             '    #end',
             '',
             '#end',
+            '',
             'print(\'\'.join(_buf))',
             ''))
         self.input = self.input_for_retrieve
@@ -397,20 +399,22 @@ class MainTest(object):
         self._test()
         #
         expected = '\n'.join((
-            '    1:  _buf = []; ',
+            '    1:  _buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; ',
             '    2:  if list:',
             '',
-            '   10:      i = 0',
-            '   11:      for item in list:',
-            '   12:  \ti += 1',
-            '   13:                     to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
-            '   14:  \t    escape(to_str(i)); ',
-            '   15:              escape(to_str(item)); ',
+            '    9:      i = 0',
+            '   10:      for item in list:',
+            '   11:          i += 1',
             '',
-            '   17:      #end',
+            '   13:          _to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
+            '   14:          _escape(_to_str(i)); ',
+            '   15:          _escape(_to_str(item)); ',
+            '',
+            '   18:      #end',
             '',
             '   20:  #end',
-            '   21:  print(\'\'.join(_buf))',
+            '',
+            '   22:  print(\'\'.join(_buf))',
             ''))
         self.expected = expected
         self.options = '-SNU'
@@ -418,14 +422,14 @@ class MainTest(object):
 
     def test_retrieve3(self):  # -SC, -SNC
         expected = '\n'.join((
-            '_buf = []; ',
+            '_buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; ',
             'if list:',
             '    i = 0',
             '    for item in list:',
-            '\ti += 1',
-            '                   to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
-            '\t    escape(to_str(i)); ',
-            '            escape(to_str(item)); ',
+            '        i += 1',
+            '        _to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
+            '        _escape(_to_str(i)); ',
+            '        _escape(_to_str(item)); ',
             '    #end',
             '#end',
             'print(\'\'.join(_buf))',
@@ -436,17 +440,17 @@ class MainTest(object):
         self._test()
         #
         expected = '\n'.join((
-            '    1:  _buf = []; ',
+            '    1:  _buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; ',
             '    2:  if list:',
-            '   10:      i = 0',
-            '   11:      for item in list:',
-            '   12:  \ti += 1',
-            '   13:                     to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
-            '   14:  \t    escape(to_str(i)); ',
-            '   15:              escape(to_str(item)); ',
-            '   17:      #end',
+            '    9:      i = 0',
+            '   10:      for item in list:',
+            '   11:          i += 1',
+            '   13:          _to_str(i % 2 and "#FFCCCC" or "#CCCCFF"); ',
+            '   14:          _escape(_to_str(i)); ',
+            '   15:          _escape(_to_str(item)); ',
+            '   18:      #end',
             '   20:  #end',
-            '   21:  print(\'\'.join(_buf))',
+            '   22:  print(\'\'.join(_buf))',
             ''))
         self.expected = expected
         self.options = '-SNC'
@@ -454,7 +458,7 @@ class MainTest(object):
 
     def test_statements(self):  # -X, -a statements
         expected = '\n'.join((
-            '_buf = []; ',
+            '_buf = []; _extend=_buf.extend;_to_str=to_str;_escape=escape; ',
             'if list:',
             '',
             '',
@@ -462,18 +466,19 @@ class MainTest(object):
             '',
             '',
             '',
-            '',
             '    i = 0',
             '    for item in list:',
-            '\ti += 1',
+            '        i += 1',
+            '',
+            '',
             '',
             '',
             '',
             '',
             '    #end',
             '',
-            '',
             '#end',
+            '',
             'print(\'\'.join(_buf))',
             ''))
         self.input = self.input_for_retrieve
@@ -508,7 +513,8 @@ class MainTest(object):
     def test_indent(self):  # -i2
         self.options  = "-si2"
         self.input    = INPUT
-        self.expected = SOURCE.replace('    _buf', '  _buf')
+        pat = re.compile(r'^    _extend', re.M)
+        self.expected = pat.sub(r'  _extend',  SOURCE)
         self._test()
 
     def test_quiet(self):  # -q, -qasyntax
@@ -626,16 +632,29 @@ class MainTest(object):
         self._test()
 
     def test_modules(self):  # -r modules
-        self.options  = "--escapefunc=cgi.escape"
-        self.input    = INPUT
-        self.expected = EXECUTED.replace('&quot;', '"')
+        #self.options  = "--escapefunc=cgi.escape"
+        #self.input    = INPUT
+        #self.expected = EXECUTED.replace('&quot;', '"')
+        #self.exception = NameError
+        #self.errormsg = "name 'cgi' is not defined"
+        #self._test()
+        ##
+        #self.options  = "-r cgi,os,sys --escapefunc=cgi.escape"
+        #self.input    = INPUT
+        #self.expected = EXECUTED.replace('&quot;', '"')
+        #self.exception = None
+        #self.errormsg = None
+        #self._test()
+        #
+        self.input    = "Hello #{cgi.escape('Haru&Kyon')}!"
+        self.expected = "Hello Haru&amp;Kyon!"
+        #
+        globals().pop('cgi', None)
         self.exception = NameError
         self.errormsg = "name 'cgi' is not defined"
         self._test()
         #
-        self.options  = "-r cgi,os,sys --escapefunc=cgi.escape"
-        self.input    = INPUT
-        self.expected = EXECUTED.replace('&quot;', '"')
+        self.options  = "-r cgi,os,sys"
         self.exception = None
         self.errormsg = None
         self._test()
@@ -649,19 +668,19 @@ class MainTest(object):
     def test_escapefunc(self):  # --escapefunc=cgi.escape
         self.options  = "-s --escapefunc=cgi.escape"
         self.input    = INPUT
-        self.expected = SOURCE.replace('escape', 'cgi.escape')
+        self.expected = SOURCE.replace('=escape', '=cgi.escape')
         self._test()
 
     def test_tostrfunc(self):  # --tostrfunc=str
         self.options  = "-s --tostrfunc=str"
         self.input    = INPUT
-        self.expected = SOURCE.replace('to_str', 'str')
+        self.expected = SOURCE.replace('=to_str', '=str')
         self._test()
 
     def test_preamble(self):  # --preamble --postamble
         self.options  = ["-s", "--preamble=_buf=list()", "--postamble=return ''.join(_buf)"]
         self.input    = INPUT
-        self.expected = re.sub(r'print\((.*?)\)', r'return \1', SOURCE).replace("_buf = []", "_buf=list()")
+        self.expected = re.sub(r'print\((.*?)\)', r'return \1', SOURCE).replace("_buf = []; ", "_buf=list(); ")
         self._test()
 
     def test_xencoding1(self):  # --encoding=encoding
@@ -820,7 +839,19 @@ class MainTest(object):
         finally:
             pass
 
+    def test_safe(self):  # --safe
+        _backup = tenjin.Engine.templateclass
+        try:
+            self.options  = "-s --safe"
+            self.input    = INPUT
+            self.input    = re.sub(r'#{(.*?)}', r'{==\1==}', self.input)
+            self.input    = re.sub(r'\${(.*?)}', r'{=\1=}', self.input)
+            self.expected = SOURCE.replace('=escape', '=to_escaped')
+            self.expected = re.sub(r'_escape\(_to_str\((.*?)\)\)', r'_escape(\1)', self.expected)
+            self._test()
+        finally:
+            tenjin.Engine.templateclass = _backup
+
 
 if __name__ == '__main__':
-    run(MainTest)
-
+    run()
