@@ -1,6 +1,6 @@
 ###
-### $Release: 0.9.0 $
-### copyright(c) 2007-2010 kuwata-lab.com all rights reserved.
+### $Release: 1.0.0 $
+### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
 ###
 
 from oktest import ok, not_ok, run
@@ -12,6 +12,8 @@ import tenjin
 #from tenjin.helpers import escape, to_str
 from tenjin.helpers import *
 
+lvars = "_extend=_buf.extend;_to_str=to_str;_escape=escape; "
+
 
 class PreprocessTest(object):
 
@@ -25,14 +27,14 @@ class PreprocessTest(object):
 	<?PY #endfor ?>
 	</select>
 	"""[1:].replace("\t", "")
-        script = r"""
+        script = lvars + r"""
 	WEEKDAY = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat']
-	_buf.extend(('''<select>
+	_extend(('''<select>
 	<?py curr = params.get(\'wday\') ?>\n''', ));
 	for i, wday in enumerate(WEEKDAY):
-	    _buf.extend(('''  <option value="''', to_str(_decode_params(i)), '''"#{selected(curr==''', to_str(_decode_params(i)), ''')}>''', escape(to_str(_decode_params(wday))), '''</option>\n''', ));
+	    _extend(('''  <option value="''', _to_str(_decode_params(i)), '''"#{selected(curr==''', _to_str(_decode_params(i)), ''')}>''', _escape(_to_str(_decode_params(wday))), '''</option>\n''', ));
 	#endfor
-	_buf.extend(('''</select>\n''', ));
+	_extend(('''</select>\n''', ));
 	"""[1:].replace("\t", "")
         preprocessed = r"""
 	<select>
@@ -58,4 +60,4 @@ class PreprocessTest(object):
 
 
 if __name__ == '__main__':
-    run(PreprocessTest)
+    run()
