@@ -1,6 +1,6 @@
 ###
-### $Release: 0.9.0 $
-### copyright(c) 2007-2010 kuwata-lab.com all rights reserved.
+### $Release: 1.0.0 $
+### $Copyright: copyright(c) 2007-2011 kuwata-lab.com all rights reserved. $
 ###
 
 from oktest import ok, not_ok, run
@@ -36,7 +36,7 @@ class MemoryBaseStoreTest(object):
 
     def test_set(self):
         data_cache, key, value = self.data_cache, self.key, self.value
-        if "called then cachees value":
+        if "called then caches value":
             ok (key).not_in(data_cache.values)
             data_cache.set(key, value, 1)
             ok (key).in_(data_cache.values)
@@ -46,10 +46,15 @@ class MemoryBaseStoreTest(object):
             t = data_cache.values[key]
             ok (t).is_a(tuple)
             ok (t[0]) == value
-            ok (int(t[1])) == int(now+10)
+            ok (int(t[1])) == int(now)
+            ok (int(t[2])) == int(now+10)
         if "called without lifetime then set cache file's mtime as 1 week ahead":
             data_cache.set(key, value)
-            ok (data_cache.values[key]) == (value, 0)
+            now = time.time()
+            t = data_cache.values[key]
+            ok (t[0]) == value
+            ok (int(t[1])) == int(now)
+            ok (t[2]) == 0
 
     def test_get(self):
         data_cache, key, value = self.data_cache, self.key, self.value
@@ -83,7 +88,7 @@ class MemoryBaseStoreTest(object):
             ok (data_cache.has(key)) == True
         if "key exists but is expired then remove cache data and returns False":
             ok (key).in_(data_cache.values)   # pre_cond
-            time.sleep(1)
+            time.sleep(1.5)
             ok (data_cache.has(key)) == False
             ok (key).not_in(data_cache.values)
 
@@ -176,6 +181,7 @@ class FragmentCacheTest(object):
 	</div>
 	"""[1:])
         self.tname = 'index.pyhtml'
+        self.tmpfiles = [self.tname]
         write_file(self.tname, pyhtml)
         self.root_dir = '_test.caches.d'
         os.mkdir(self.root_dir)
@@ -190,7 +196,10 @@ class FragmentCacheTest(object):
         tenjin.helpers.fragment_cache.prefix = 'fragment.'
 
     def after(self):
-        os.unlink(self.tname)
+        for tmpfile in self.tmpfiles:
+            for fname in [tmpfile, tmpfile + '.cache']:
+                if os.path.exists(fname):
+                    os.unlink(fname)
         shutil.rmtree(self.root_dir)
         tenjin.helpers.fragment_cache.store  = self._orig_store
         tenjin.helpers.fragment_cache.prefix = self._orig_preifx
@@ -245,6 +254,59 @@ class FragmentCacheTest(object):
         ok (tupl[0]) == fragment_cache.not_cached
         ok (tupl[1]) == fragment_cache.echo_cached
 
+    def test_cache_as(self):
+        input = r"""
+<div>
+<?py for _ in cache_as('items/123', 2): ?>
+  <ul>
+    <?py for item in items: ?>
+    <li>${item}</li>
+    <?py #endfor ?>
+  </ul>
+<?py #endfor ?>
+</div>
+"""[1:]
+        expected_fragment = r"""
+  <ul>
+    <li>A</li>
+    <li>B</li>
+  </ul>
+"""[1:]
+        expected_html = "<div>\n" + expected_fragment + "</div>\n"
+        file_name = "_test_cache_as.pyhtml"
+        f = open(file_name, "w")
+        f.write(input)
+        f.close()
+        engine = tenjin.Engine()
+        self.tmpfiles.append(file_name)
+        fragment_cache_path = self.root_dir + '/fragment.items/123'
+        #
+        ts = None
+        if "called at first time then cache fragment into file":
+            context = {'items': ['A','B']}
+            output = engine.render(file_name, context)
+            ok (output) == expected_html
+            ok (fragment_cache_path).is_file()
+            ok (_read_file(fragment_cache_path)) == expected_fragment
+            ts = os.path.getmtime(fragment_cache_path)
+        if "called at second time within lifetime then dont't render":
+            #now = time.time()
+            #os.utime(fragment_cache_path, (now-1, now-1))
+            time.sleep(1)
+            context = {'items': ['X','Y']}   # new context data
+            output = engine.render(file_name, context)
+            ok (output) == expected_html     # not changed
+            ok (os.path.getmtime(fragment_cache_path)) == ts
+        if "called after lifetime expired then render again":
+            #os.utime(fragment_cache_path, (now-3, now-3))
+            time.sleep(2)
+            context = {'items': ['X','Y']}   # new context data
+            output = engine.render(file_name, context)
+            edit = lambda s: s.replace('A', 'X').replace('B', 'Y')
+            ok (output) == edit(expected_html)   # changed!
+            ok (_read_file(fragment_cache_path)) == edit(expected_fragment)  # changed!
+            ok (os.path.getmtime(fragment_cache_path)) > ts
+
 
 if __name__ == '__main__':
-    run(MemoryBaseStoreTest, FileBaseStoreTest, FragmentCacheTest)
+    run()
